classdef list
% list - forward-linked list 
    
    properties (SetAccess = protected, GetAccess = protected)
        elements = {};
        current = 0;
    end
    
    methods
        function L = list(varargin)
        % list constructor
        % L = list(x1, x2, x3, ...) creates a list object and initializes 
        % the list with the input arguments.
        % If no input arguments are given, an empty list is returned.
            if nargin > 0
                for Index = 1:nargin
                    L.elements{Index} = varargin{Index};
                end
                L.current = 1;
            end
        end
        
        function E = get_current(L)
        % get_current(L) - returns the element currently pointed to in list L
            if L.current > 0
                E = L.elements{L.current};
            end
        end
        
        function L = reverse(L)
        % reverse(L) - reverses the elements of list L
            Len = length(L.elements);
            for Index = 1:Len/2
                Temp = L.elements{Index};
                L.elements{Index} = L.elements{Len - Index + 1};
                L.elements{Len - Index + 1} = Temp;
            end
        end
        
        function S = size(L, varargin)
        % size(L) - returns the number of elements in list L
        %         - varargin is used to catch other arguments and ignore them.
            S = length(L.elements);
        end
        
        function L = append(L, varargin)
        % append - appends one or more new elements to list L
            for Index = 1:nargin-1
                numElements = L.size();
                if isequal(numElements, 0)
                    L.current = 1;
                end
                L.elements{numElements + 1} = varargin{Index};
            end
        end
        
        function display(L)
        % Display - displays a list object L

            % get the number of elements in the list
            Len = length(L.elements);

            % display the name of the list
            disp(' ');
            disp([inputname(1), ' =']);

            % loop through each element and display it
            Str = '';
            for Index = 1:Len
                % get size of each element
                if isequal(size(L.elements{Index}, 1), 1)
                    elSize = length(L.elements{Index});
                else
                    elSize = size(L.elements{Index});
                end
                % display element
                disp(['   ', num2str(Index), ': ', ...
                     class(L.elements{Index}), ...
                     '(', num2str(elSize), ')']);
                disp(L.elements{Index})
            end        
        end
        
        function L = first(L)
        % first(L) - returns a list L where the first element is the current element
            if length(L.elements) > 0
                L.current = 1;
            end
        end
        
        function L = next(L)
        % next(L) - returns a list L where the next element is the current element
            if L.current < length(L.elements)
                L.current = L.current + 1;
            end        
        end
        
        function L1 = plus(L1, L2)
        % plus - use the + operator to concatenate two lists
            for Index = 1:length(L2.elements)
                numElements = length(L1.elements);
                L1.elements{numElements + 1} = L2.elements{Index};
            end
        end
        
        function L = remove(L, E)
        % remove(L, E) - deletes the first occurrence of element E from list L
            for Index=1:length(L.elements)
                if isequal(E, L.elements{Index})
                    L.elements{Index} = {};
                    if Index >= L.current
                        L.current = L.current - 1;
                    end
                    return;
                end
            end
        end
    end
end

